@@ -1914,9 +1914,17 @@ catchXMLError(void *ctx, const char *msg ATTRIBUTE_UNUSED, ...)
     if (ctxt && ctxt->_private) {
         GError **err = ctxt->_private;
         if (!error_is_set(err)) {
-            gchar *xmlmsg = g_strdup_printf("at line %d: %s",
-                                            ctxt->lastError.line,
-                                            ctxt->lastError.message);
+            gchar *xmlmsg;
+            if (ctxt->lastError.file) {
+                xmlmsg = g_strdup_printf("%s:%d: %s",
+                                         ctxt->lastError.file,
+                                         ctxt->lastError.line,
+                                         ctxt->lastError.message);
+            } else {
+                xmlmsg = g_strdup_printf("at line %d: %s",
+                                         ctxt->lastError.line,
+                                         ctxt->lastError.message);
+            }
             OSINFO_LOADER_SET_ERROR(ctxt->_private, xmlmsg);
             g_free(xmlmsg);
         }
@@ -2168,6 +2176,8 @@ osinfo_loader_process_file_reg_xml(OsinfoLoader *loader,
     if (error_is_set(err))
         return;
 
+    if (xmlLen == 0)
+        return;
     if (base) {
         relpath = g_file_get_relative_path(base, file);
         if (relpath == NULL) {
@@ -2338,11 +2348,19 @@ static void osinfo_loader_find_files(OsinfoLoader *loader,
             ent = g_file_get_child(file, name);
             type = g_file_info_get_attribute_uint32(info,
                                                     G_FILE_ATTRIBUTE_STANDARD_TYPE);
-            if (type == G_FILE_TYPE_REGULAR) {
-                if (g_str_has_suffix(name, ".xml"))
+            if (type == G_FILE_TYPE_DIRECTORY) {
+                if (!g_str_equal(name, "schema"))
+                    osinfo_loader_find_files(loader, base, ent, entries, FALSE, &error);
+            } else {
+                if (g_str_has_suffix(name, ".xml")) {
                     osinfo_loader_entity_files_add_path(entries, base, ent);
-            } else if (type == G_FILE_TYPE_DIRECTORY) {
-                osinfo_loader_find_files(loader, base, ent, entries, FALSE, &error);
+                } else if (!g_str_equal(name, "LICENSE") &&
+                           !g_str_equal(name, "VERSION") &&
+                           !g_str_has_suffix(name, "~") &&
+                           !g_str_has_suffix(name, ".bak")) {
+                    g_autofree gchar *path = g_file_get_path(ent);
+                    g_printerr("Ignoring %s with missing '.xml' extension\n", path);
+                }
             }
             g_object_unref(ent);
             g_object_unref(info);
@@ -2356,6 +2374,26 @@ static void osinfo_loader_find_files(OsinfoLoader *loader,
         }
         g_object_unref(ents);
         g_list_free(children);
+    } else if (type == G_FILE_TYPE_UNKNOWN) {
+        g_autofree gchar *path = g_file_get_path(file);
+        g_autofree gchar *msg = g_strdup_printf("Can't read path %s", path);
+        if (skipMissing) {
+            /* This is a work-around for
+             * <https://gitlab.gnome.org/GNOME/glib/-/issues/1237>. If the
+             * lstat() call underlying our g_file_query_info() call at the top
+             * of this function fails for "path" with EACCES, then
+             * g_file_query_info() should fail, and the "skipMissing" branch up
+             * there should suppress the error and return cleanly.
+             * Unfortunately, _g_local_file_info_get() masks the lstat()
+             * failure, g_file_info_get_attribute_uint32() is reached above,
+             * and returns G_FILE_TYPE_UNKNOWN for the file that could never be
+             * accessed. So we need to consider "skipMissing" here too.
+             */
+            g_warning("%s", msg);
+            return;
+        }
+        OSINFO_LOADER_SET_ERROR(&error, msg);
+        g_propagate_error(err, error);
     } else {
         OSINFO_LOADER_SET_ERROR(&error, "Unexpected file type");
         g_propagate_error(err, error);
@@ -2486,7 +2524,7 @@ static void osinfo_loader_process_list(OsinfoLoader *loader,
                                                files->master, &lerr);
             if (lerr) {
                 g_propagate_error(err, lerr);
-                break;
+                goto cleanup;
             }
         }
 
@@ -2499,7 +2537,7 @@ static void osinfo_loader_process_list(OsinfoLoader *loader,
                                                &lerr);
             if (lerr) {
                 g_propagate_error(err, lerr);
-                break;
+                goto cleanup;
             }
 
             tmpl = tmpl->next;
